(ns com.aurellem.vbm
  (:import java.io.File)
  (:import org.apache.commons.io.FileUtils))

(defn vbm-bytes [#^File vbm]
  (let [bytes (FileUtils/readFileToByteArray vbm)
        ints (int-array (count bytes))]
    (areduce bytes idx _ nil
             (aset ints idx
                   (bit-and 0xFF (aget bytes idx))))
    ints))

(def button-mask
  {;; main buttons
   :a         0x0001
   :b         0x0002

   ;; directional pad
   :r         0x0010
   :l         0x0020
   :u         0x0040
   :d         0x0080

   ;; meta buttons
   :select    0x0004
   :start     0x0008

   ;; hard reset -- not really a button
   :reset   0x0800})

(defn button-code [buttons]
  (reduce bit-or 0x0000 (map button-mask buttons)))

(defn buttons [mask]
  (loop [buttons []
         masks (seq button-mask)]
    (if (empty? masks) buttons
        (let [[button value] (first masks)]
          (if (not= 0x0000 (bit-and value mask))
            (recur (conj buttons button) (rest masks))
            (recur buttons (rest masks)))))))

(def vbm-header-length 255)

(defn repair-vbm
  "Two 0's must be inserted after every reset, and the first
   button must be dropped"
  [vbm-seq]
  (loop [fixed []
         pending (next vbm-seq)]
    (if (empty? pending) fixed
        (let [mask (first pending)]
          (if (not= 0x0000 (bit-and mask (button-mask :reset)))
            (recur (conj fixed mask 0x0000 0x0000) (next pending))
            (recur (conj fixed mask) (next pending)))))))

(defn vbm-masks [#^File vbm]
  (repair-vbm
   (map (fn [[a b]]
          (+ (bit-shift-left a 8) b))
        (partition
         2 (drop vbm-header-length (vbm-bytes vbm))))))

(defn vbm-buttons [#^File vbm]
  (map buttons (vbm-masks vbm)))
  


         
    
  